@@ -1,10 +1,15 @@
 use utf8;
 use strict;
 
+
+#=======================================================
+package XML::Pastor::Schema;
+
 use XML::Pastor::Schema::Attribute;
 use XML::Pastor::Schema::AttributeGroup;
 use XML::Pastor::Schema::ComplexType;
 use XML::Pastor::Schema::Context;
+use XML::Pastor::Schema::Documentation;
 use XML::Pastor::Schema::Element;
 use XML::Pastor::Schema::Group;
 use XML::Pastor::Schema::List;
@@ -15,7 +20,6 @@ use XML::Pastor::Schema::Parser;
 use XML::Pastor::Schema::SimpleType;
 use XML::Pastor::Schema::Union;
 
-package XML::Pastor::Schema;
 our @ISA = qw(XML::Pastor::Schema::Object);
 
 XML::Pastor::Schema->mk_accessors(
